#version 330
#extension GL_ARB_shading_language_420pack : require

// Uniform
uniform float f_thickness;		// width of the line
uniform float f_halo;			// width of the halo
uniform float f_stripDepth;		// depth displacement at halo edges
uniform float f_depthCueing;	// amount of depth cueing
uniform mat4 m_projection;

// Textures
layout (binding = 0) uniform sampler2D textureStroke;
layout (binding = 1) uniform sampler2D textureStripMask;

// Input
in VertexData
{
    vec2 UV;
	vec4 posVS;

} inData;

// Output
layout (location = 0) out vec4 fragmentColor;

// Returns screen space depth (which is written to the depth buffer)
// given the depth in view space
float getScreenSpaceDepth(float zVS)
{
    float zNDC = -m_projection[2][2] - (m_projection[3][2] / zVS);

    float near = gl_DepthRange.near;
	float far = gl_DepthRange.far;

    return (((far - near) * zNDC) + near + far) / 2.0;
}

void main()
{
    // Tapering
	if (texture(textureStripMask, inData.UV).x < 0.5)
	{
	    discard;
	}

    float side = abs(2 * (inData.UV.y - 0.5));

    // Depth cueing
	float line = f_thickness / (f_thickness + f_halo);
        //line *= (1 - f_depthCueing) + f_depthCueing * (1 + inData.posVS.z);
        line *= ((1 - f_depthCueing) + f_depthCueing * ((3.0+inData.posVS.z)*0.5));

    // Sample texture to get color (v < 0.5 = black, v > 0.5 = white)
	float tc = side - line + 0.5;
	float stroke = texture(textureStroke, vec2(inData.UV.x, tc)).x;

    // Displace in view space and adjust fragment depth
    float displacement = float(stroke > 0) * side * f_stripDepth;
    gl_FragDepth = getScreenSpaceDepth(inData.posVS.z - displacement);

        fragmentColor = vec4(stroke, stroke, stroke, 1);
}
