#include "camera.h"

Camera::Camera(QObject* parent, float minDist, float maxDist, float initDist, float initYaw, float initPitch)
	: QObject(parent), m_aspectRatio(1), m_near(0.1f), m_far(100.0f)
{
	m_minDist = qMax(.0f, minDist);
	m_maxDist = qMax(m_minDist, maxDist);
	m_dist = qMin(m_maxDist, qMax(m_minDist, initDist));

	const float offset = 0.01f;
	m_xRot = std::fmod(initYaw, 2 * Math::PI);
	m_yRot = qMax(offset, qMin(Math::PI - offset, initPitch));
}

void Camera::setAspectRatio(float aspectRatio)
{
	m_aspectRatio = aspectRatio;
}

void Camera::setNearFarPlane(float near, float far)
{
	m_near = near;
	m_far = far;
}

void Camera::rotateX(float angle)
{
	m_xRot = std::fmod(m_xRot + angle, 2 * Math::PI);
}

void Camera::rotateY(float angle)
{
	const float offset = 0.01f;
	m_yRot = qMax(offset, qMin(Math::PI - offset, m_yRot + angle));
}

void Camera::move(const QVector2D& offset)
{
	m_offset += offset;
}

void Camera::zoom(float distance)
{
    m_dist = qMin(m_maxDist, qMax(m_minDist, m_dist + distance));
}

void Camera::getViewMatrix(QMatrix4x4& viewMatrix)
{
	viewMatrix.setToIdentity();

	// Orbital Camera
	float phi = m_xRot;
	float theta = m_yRot;
    float r = 2.0; //m_dist;

	QVector3D eye = QVector3D(r * cos(phi) * sin(theta), r * cos(theta) , r * sin(phi) * sin(theta));
	QVector3D target = QVector3D(0,0,0);
	QVector3D up = QVector3D(0,1,0);

	QVector3D offset1 = QVector3D::crossProduct(up, (target-eye).normalized()).normalized();
	QVector3D offset2 = QVector3D::crossProduct(offset1, (target-eye).normalized()).normalized()*m_offset.y();
	offset1 *= m_offset.x();

	target = target + offset1+offset2;

	viewMatrix.lookAt(target+eye, target, up);
}

void Camera::getProjectionMatrix(QMatrix4x4& projMatrix)
{
	projMatrix.setToIdentity();
    projMatrix.perspective(m_dist, m_aspectRatio, m_near, m_far);
}
