#ifndef TRACKFILE_H
#define TRACKFILE_H

#include <QString>
#include <QVector>
#include <QVector3D>

/**
 * @brief A vector of QVector3D representing the points of a track.
 */
typedef QVector<QVector3D> Track;

/**
 * @brief Static class for reading trk files.
 */
class TrackFile
{
	public:
		/**
		 * @brief Reads track data from a trk file.
		 * @param fname the file to read.
		 * @param tracks returns the read data.
		 * @return true on success, false otherwise.
		 *
		 * This function reads a data file of the format specified <a href="http://trackvis.org/docs/?subsect=fileformat">here</a>.
		 * On success, true is returned and the read data is stored in @p tracks. On failure, false is returned and the content of @p tracks
		 * is undefined.
		 */
		static bool read(const QString& fname, QVector<Track>& tracks);
};

#endif // TRACKFILE_H
